@@ -15,58 +15,87 @@
 #' @details
 #'  Because of complexity and multimodality of the log-likelihood function, it's \strong{not certain} that the estimation
 #'  algorithms will end up in the global maximum point. It's expected that most of the estimation rounds will end up in
-#'  some local maximum point instead. Therefore a number of estimation rounds is required for reliable results. Because
-#'  of the nature of the model, the estimation may fail especially in the cases where the number of mixture components
-#'  is chosen too large.
+#'  some local maximum or saddle point instead. Therefore, a (sometimes large) number of estimation rounds is required
+#'  for reliable results. Because of the nature of the model, the estimation may fail especially in the cases where the
+#'  number of mixture components is chosen too large.
 #'
 #'  The estimation process is computationally heavy and it might take considerably long time for large models with
 #'  large number of observations. If the iteration limit \code{maxit} in the variable metric algorithm is reached,
 #'  one can continue the estimation by iterating more with the function \code{iterate_more}. Alternatively, one may
-#'  use the found estimates as starting values for the genetic algorithm and and employ another round of estimation.
+#'  use the found estimates as starting values for the genetic algorithm and and employ another round of estimation
+#'  (see \code{?GAfit} how to set up an initial population with the dot parameters).
 #'
 #'  The code of the genetic algorithm is mostly based on the description by \emph{Dorsey and Mayer (1995)} but it
 #'  includes some extra features that were found useful for this particular estimation problem. For instance,
 #'  the genetic algorithm uses a slightly modified version of the individually adaptive crossover and mutation
-#'  rates descriped by \emph{Patnaik and Srinivas (1994)} and employs (50\%) fitness inheritance discussed
+#'  rates described by \emph{Patnaik and Srinivas (1994)} and employs (50\%) fitness inheritance discussed
 #'  by \emph{Smith, Dike and Stegmann (1995)}.
 #'
 #'  The gradient based variable metric algorithm used in the second phase is implemented with function \code{optim}
 #'  from the package \code{stats}.
-#' @return Returns an object of class \code{'gmvar'} defining the estimated GMVAR model. Multivariate quantile residuals
-#'   (Kalliovirta and Saikkonen 2010) are also computed and included in the returned object. In addition, the returned
-#'   object contains the estimates and log-likelihood values from all the estimation rounds performed.
+#'
+#'  Finally, note that the structural models are even more difficult to estimate than the reduced form models due to
+#'  the different parametrization of the covariance matrices. If necessary, an initial population may be constructed
+#'  for the genetic algorithm based on the estimation results of a reduced form model. It does not seem unambiguous,
+#'  however, how to do that so that the (structural) parameter constraints are satisfied. Also, be aware that if the
+#'  lambda parameters are constrained in any other way than by restricting some of them to be identical, the parameter
+#'  "lambda_scale" of the genetic algorithm (see \code{?GAfit}) needs to be carefully adjusted accordingly.
+#'  Be aware that if a structural model is considered and the lambda parameters are constrained in some other way
+#'  than constraining some of them to be identical, the settings of the genetic algorithm may have to be adjusted.
+#' @return Returns an object of class \code{'gmvar'} defining the estimated (reduced form or structural) GMVAR model.
+#'   Multivariate quantile residuals (Kalliovirta and Saikkonen 2010) are also computed and included in the returned object.
+#'   In addition, the returned object contains the estimates and log-likelihood values from all the estimation rounds performed.
 #'   The estimated parameter vector can be obtained at \code{gmvar$params} (and corresponding approximate standard errors
 #'   at \code{gmvar$std_errors}) and it is...
 #'   \describe{
-#'     \item{\strong{Unconstrained models:}}{
-#'       a size \eqn{((M(pd^2+d+d(d+1)/2+1)-1)x1)} vector that has form
+#'     \item{\strong{For unconstrained models:}}{
+#'       ...a size \eqn{((M(pd^2+d+d(d+1)/2+1)-1)x1)} vector that has form
 #'       \strong{\eqn{\theta}}\eqn{ = }(\strong{\eqn{\upsilon}}\eqn{_{1}},
-#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1}}), where:
+#'       ...,\strong{\eqn{\upsilon}}\eqn{_{M}}, \eqn{\alpha_{1},...,\alpha_{M-1}}), where
 #'       \itemize{
 #'         \item \strong{\eqn{\upsilon}}\eqn{_{m}} \eqn{ = (\phi_{m,0},}\strong{\eqn{\phi}}\eqn{_{m}}\eqn{,\sigma_{m})}
 #'         \item \strong{\eqn{\phi}}\eqn{_{m}}\eqn{ = (vec(A_{m,1}),...,vec(A_{m,p})}
 #'         \item and \eqn{\sigma_{m} = vech(\Omega_{m})}, m=1,...,M.
 #'       }
 #'     }
-#'     \item{\strong{Constrained models:}}{
-#'       a size \eqn{((M(d+d(d+1)/2+1)+q-1)x1)} vector that has form
+#'     \item{\strong{For constrained models:}}{
+#'       ...a size \eqn{((M(d+d(d+1)/2+1)+q-1)x1)} vector that has form
 #'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\psi}}
-#'       \eqn{,\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1})}, where:
+#'       \eqn{,\sigma_{1},...,\sigma_{M},\alpha_{1},...,\alpha_{M-1})}, where
 #'       \itemize{
 #'         \item \strong{\eqn{\psi}} \eqn{(qx1)} satisfies (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
-#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}}. Here \strong{\eqn{C}} is \eqn{(Mpd^2xq)}
+#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}} where \strong{\eqn{C}} is \eqn{(Mpd^2xq)}
 #'         constraint matrix.
 #'       }
 #'     }
+#'     \item{\strong{For structural GMVAR model:}}{
+#'       ...a vector that has the form
+#'       \strong{\eqn{\theta}}\eqn{ = (\phi_{1,0},...,\phi_{M,0},}\strong{\eqn{\phi}}\eqn{_{1},...,}\strong{\eqn{\phi}}\eqn{_{M},
+#'       vec(W),}\strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M},\alpha_{1},...,\alpha_{M-1})}, where
+#'       \itemize{
+#'         \item\strong{\eqn{\lambda}}\eqn{_{m}=(\lambda_{m1},...,\lambda_{md})} contains the eigenvalues of the \eqn{m}th mixture component.
+#'       }
+#'       \describe{
+#'         \item{\strong{If AR parameters are constrained: }}{Replace \strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
+#'         \strong{\eqn{\phi}}\eqn{_{M}} with \strong{\eqn{\psi}} \eqn{(qx1)} that satisfies (\strong{\eqn{\phi}}\eqn{_{1}}\eqn{,...,}
+#'         \strong{\eqn{\phi}}\eqn{_{M}) =} \strong{\eqn{C \psi}}, as above.}
+#'         \item{\strong{If \eqn{W} is constrained:}}{Remove the zeros from \eqn{vec(W)} and make sure the other entries satisfy
+#'          the sign constraints.}
+#'         \item{\strong{If \eqn{\lambda_{mi}} are constrained:}}{Replace \strong{\eqn{\lambda}}\eqn{_{2},...,}\strong{\eqn{\lambda}}\eqn{_{M}}
+#'          with \strong{\eqn{\gamma}} \eqn{(rx1)} that satisfies (\strong{\eqn{\lambda}}\eqn{_{2}}\eqn{,...,}
+#'         \strong{\eqn{\lambda}}\eqn{_{M}) =} \strong{\eqn{C_{\lambda} \gamma}} where \eqn{C_{\lambda}} is a \eqn{(d(M-1) x r)}
+#'          constraint matrix.}
+#'       }
+#'     }
 #'   }
-#'   Above, \eqn{\phi_{m,0}} is the intercept parameter, \eqn{A_{m,i}} denotes the \eqn{i}:th coefficient matrix of
-#'   the \eqn{m}:th mixture component, \eqn{\Omega_{m}} denotes the error term covariance matrix of the \eqn{m}:th
-#'   mixture component, and \eqn{\alpha_{m}} is the mixing weight parameter.
+#'   Above, \eqn{\phi_{m,0}} is the intercept parameter, \eqn{A_{m,i}} denotes the \eqn{i}th coefficient matrix of the \eqn{m}th
+#'   mixture component, \eqn{\Omega_{m}} denotes the error term covariance matrix of the \eqn{m}:th mixture component, and
+#'   \eqn{\alpha_{m}} is the mixing weight parameter. The \eqn{W} and \eqn{\lambda_{mi}} are structural parameters replacing the
+#'   error term covariance matrices (see Virolainen, 2020). If \eqn{M=1}, \eqn{\alpha_{m}} and \eqn{\lambda_{mi}} are dropped.
 #'   If \code{parametrization=="mean"}, just replace each \eqn{\phi_{m,0}} with regimewise mean \eqn{\mu_{m}}.
-#'   \eqn{vec()} is vectorization operator that stacks columns of a given matrix into a vector. \eqn{vech()} stacks
-#'   columns of a given matrix from the principal diagonal downwards (including elements on the diagonal) into a vector.
-#'   The notation is in line with the cited article by \emph{Kalliovirta, Meitz and Saikkonen (2016)} which introduces
-#'   the GMVAR model.
+#'   \eqn{vec()} is vectorization operator that stacks columns of a given matrix into a vector. \eqn{vech()} stacks columns
+#'   of a given matrix from the principal diagonal downwards (including elements on the diagonal) into a vector.
+#'   The notation is in line with the cited article by \emph{Kalliovirta, Meitz and Saikkonen (2016)} introducing the GMVAR model.
 #'
 #'   Remark that the first autocovariance/correlation matrix in \code{$uncond_moments} is for the lag zero,
 #'   the second one for the lag one, etc.
@@ -75,7 +104,8 @@
 #'    \code{predict} and \code{plot}.
 #' @seealso \code{\link{GMVAR}}, \code{\link{iterate_more}}, \code{\link{predict.gmvar}}, \code{\link{profile_logliks}},
 #'   \code{\link{simulateGMVAR}}, \code{\link{quantile_residual_tests}}, \code{\link{print_std_errors}},
-#'   \code{\link{swap_parametrization}}, \code{\link{get_gradient}}
+#'   \code{\link{swap_parametrization}}, \code{\link{get_gradient}}, \code{\link{GIRF}}, \code{\link{LR_test}}, \code{\link{Wald_test}},
+#'   \code{\link{gmvar_to_sgmvar}}, \code{\link{reorder_W_columns}}, \code{\link{swap_W_signs}}
 #' @references
 #'  \itemize{
 #'    \item Dorsey R. E. and Mayer W. J. 1995. Genetic algorithms for estimation problems with multiple optima,
@@ -89,6 +119,8 @@
 #'          \emph{Transactions on Systems, Man and Cybernetics} \strong{24}, 656-667.
 #'    \item Smith R.E., Dike B.A., Stegmann S.A. 1995. Fitness inheritance in genetic algorithms.
 #'          \emph{Proceedings of the 1995 ACM Symposium on Applied Computing}, 345-350.
+#'    \item Virolainen S. 2020. Structural Gaussian mixture vector autoregressive model. Unpublished working
+#'      paper, available as arXiv:2007.04713.
 #'  }
 #' @examples
 #' \donttest{
@@ -98,7 +130,7 @@
 #' # package, but in a scaled form (similar to Kalliovirta et al. 2016).
 #' data(eurusd, package="gmvarkit")
 #' data <- cbind(10*eurusd[,1], 100*eurusd[,2])
-#' colnames(dat) <- colnames(eurusd)
+#' colnames(data) <- colnames(eurusd)
 #'
 #' # GMVAR(1,2) model: 10 estimation rounds with seeds set
 #' # for reproducibility
@@ -106,12 +138,35 @@
 #' fit12
 #' plot(fit12)
 #' summary(fit12)
+#' print_std_errors(fit12)
+#' profile_logliks(fit12)
+#'
+#' # Structural GMVAR(1,2) model identified with sign
+#' # constraints. The sign constraints (which fully identify
+#' # the shocks) are in line with the reduced form model,
+#' # so the maximized loglikelihood is the same.
+#' W_122 <- matrix(c(1, NA, -1, 1), nrow=2)
+#' fit12s <- fitGMVAR(data, p=1, M=2, structural_pars=list(W=W_122),
+#'   ncalls=10, seeds=1:10)
+#' fit12s
 #'
 #' # GMVAR(2,2) model with mean parametrization
 #' fit22 <- fitGMVAR(data, p=2, M=2, parametrization="mean",
 #'                   ncalls=16, seeds=1:16)
 #' fit22
 #'
+#' # Structural GMVAR(2,2) model with the lambda parameters restricted
+#' # to be identical (in the second regime) and the shocks identified
+#' # with diagonal of the B-matrix normalized positive and one zero constraint.
+#' # The resulting model has error term covariance matrices that are
+#' # multiplicatives of each other, while the identification equals to
+#' # identification through Cholesky decomposition.
+#' W_222 <- matrix(c(1, NA, 0, 1), nrow=2)
+#' C_lambda_222 <- matrix(c(1, 1), nrow=2)
+#' fit22s <- fitGMVAR(data, p=2, M=2, structural_pars=list(W=W_222, C_lambda=C_lambda_222),
+#'   ncalls=20, seeds=1:20)
+#' fit22s
+#'
 #' # GMVAR(2,2) model with autoregressive parameters restricted
 #' # to be the same for both regimes
 #' C_mat <- rbind(diag(2*2^2), diag(2*2^2))
@@ -130,17 +185,19 @@
 #' }
 #' @export
 
-fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept", "mean"), constraints=NULL, ncalls=round(10 + 9*log(M)),
-                     ncores=min(2, ncalls, parallel::detectCores()), maxit=300, seeds=NULL, print_res=TRUE, ...) {
+fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept", "mean"), constraints=NULL, structural_pars=NULL,
+                     ncalls=round(10 + 9*log(M)), ncores=min(2, ncalls, parallel::detectCores()), maxit=300, seeds=NULL,
+                     print_res=TRUE, ...) {
 
   on.exit(closeAllConnections())
   if(!all_pos_ints(c(p, M, ncalls, ncores, maxit))) stop("Arguments p, M, ncalls, ncores, and maxit must be positive integers")
+  stopifnot(length(ncalls) == 1)
   if(!is.null(seeds) && length(seeds) != ncalls) stop("The argument 'seeds' needs be NULL or a vector of length 'ncalls'")
   parametrization <- match.arg(parametrization)
   data <- check_data(data=data, p=p)
   d <- ncol(data)
   n_obs <- nrow(data)
-  npars <- n_params(p=p, M=M, d=d, constraints=constraints)
+  npars <- n_params(p=p, M=M, d=d, constraints=constraints, structural_pars=structural_pars)
   if(npars >= d*nrow(data)) stop("There are at least as many parameters in the model as there are observations in the data")
   dot_params <- list(...)
   minval <- ifelse(is.null(dot_params$minval), get_minval(data), dot_params$minval)
@@ -161,13 +218,14 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
   parallel::clusterExport(cl, ls(environment(fitGMVAR)), envir = environment(fitGMVAR)) # assign all variables from package:gmvarkit
   parallel::clusterEvalQ(cl, c(library(Brobdingnag), library(mvnfast), library(pbapply)))
 
-  cat("Optimizing with the genetic algorithm...\n")
+  cat("Optimizing with a genetic algorithm...\n")
   GAresults <- pbapply::pblapply(1:ncalls, function(i1) GAfit(data=data, p=p, M=M, conditional=conditional, parametrization=parametrization,
-                                                              constraints=constraints, seed=seeds[i1], ...), cl=cl)
+                                                              constraints=constraints, structural_pars=structural_pars, seed=seeds[i1], ...), cl=cl)
 
   loks <- vapply(1:ncalls, function(i1) loglikelihood_int(data, p, M, params=GAresults[[i1]], conditional=conditional,
                                                           parametrization=parametrization, constraints=constraints,
-                                                          check_params=TRUE, to_return="loglik", minval=minval), numeric(1))
+                                                          structural_pars=structural_pars, check_params=TRUE,
+                                                          to_return="loglik", minval=minval), numeric(1))
 
   if(print_res) {
     print_loks <- function() {
@@ -176,14 +234,15 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
       printfun("The mean loglik:   ", mean)
       printfun("The largest loglik:", max)
     }
-    cat("Results from genetic algorithm:\n")
+    cat("Results from the genetic algorithm:\n")
     print_loks()
   }
 
   ### Optimization with the variable metric algorithm###
   loglik_fn <- function(params) {
     tryCatch(loglikelihood_int(data, p, M, params=params, conditional=conditional, parametrization=parametrization,
-                               constraints=constraints, check_params=TRUE, to_return="loglik", minval=minval), error=function(e) minval)
+                               constraints=constraints, structural_pars=structural_pars, check_params=TRUE,
+                               to_return="loglik", minval=minval), error=function(e) minval)
   }
 
   h <- 6e-6
@@ -192,7 +251,7 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
     vapply(1:npars, function(i1) (loglik_fn(params + I[i1,]*h) - loglik_fn(params - I[i1,]*h))/(2*h), numeric(1))
   }
 
-  cat("Optimizing with variable metric algorithm...\n")
+  cat("Optimizing with a variable metric algorithm...\n")
   NEWTONresults <- pbapply::pblapply(1:ncalls, function(i1) optim(par=GAresults[[i1]], fn=loglik_fn, gr=loglik_grad, method="BFGS",
                                                                   control=list(fnscale=-1, maxit=maxit)), cl=cl)
   parallel::stopCluster(cl=cl)
@@ -200,8 +259,9 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
   converged <- vapply(1:ncalls, function(i1) NEWTONresults[[i1]]$convergence == 0, logical(1))
 
   loks <- vapply(1:ncalls, function(i1) loglikelihood_int(data=data, p=p, M=M, params=NEWTONresults[[i1]]$par, conditional=conditional,
-                                                          constraints=constraints, parametrization=parametrization, check_params=TRUE,
-                                                          to_return="loglik", minval=minval), numeric(1))
+                                                          parametrization=parametrization, constraints=constraints,
+                                                          structural_pars=structural_pars, check_params=TRUE, to_return="loglik",
+                                                          minval=minval), numeric(1))
   if(print_res) {
     cat("Results from the variable metric algorithm:\n")
     print_loks()
@@ -212,16 +272,17 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
   all_estimates <- lapply(NEWTONresults, function(x) x$par)
   best_fit <- NEWTONresults[[which(loks == max(loks))[1]]]
   params <- best_fit$par
-  if(is.null(constraints)) {
-    params <- sort_components(p=p, M=M, d=d, params=params)
-    all_estimates <- lapply(all_estimates, function(pars) sort_components(p=p, M=M, d=d, params=pars))
+  if(is.null(constraints) && is.null(structural_pars$C_lambda)) {
+    params <- sort_components(p=p, M=M, d=d, params=params, structural_pars=structural_pars)
+    all_estimates <- lapply(all_estimates, function(pars) sort_components(p=p, M=M, d=d, params=pars, structural_pars=structural_pars))
   }
   if(best_fit$convergence == 1) {
-    message("Iteration limit was reached when estimating the best fitting individual! Consider further estimations with the function 'iterate_more'")
+    message("Iteration limit was reached when estimating the best fitting individual! Consider further estimation with the function 'iterate_more'")
   }
   mixing_weights <- loglikelihood_int(data=data, p=p, M=M, params=params, conditional=conditional,
                                       parametrization=parametrization, constraints=constraints,
-                                      to_return="mw", check_params=TRUE, minval=NULL)
+                                      structural_pars=structural_pars, to_return="mw", check_params=TRUE,
+                                      minval=NULL)
   if(any(vapply(1:M, function(i1) sum(mixing_weights[,i1] > red_criteria[1]) < red_criteria[2]*n_obs, logical(1)))) {
     message("At least one of the mixture components in the estimated model seems to be wasted!")
   }
@@ -230,7 +291,7 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
   ### Wrap up ###
   cat("Calculating approximate standard errors...\n")
   ret <- GMVAR(data=data, p=p, M=M, d=d, params=params, conditional=conditional, parametrization=parametrization,
-               constraints=constraints, calc_std_errors=TRUE)
+               constraints=constraints, structural_pars=structural_pars, calc_std_errors=TRUE)
   ret$all_estimates <- all_estimates
   ret$all_logliks <- loks
   ret$which_converger <- converged
@@ -274,6 +335,17 @@ fitGMVAR <- function(data, p, M, conditional=TRUE, parametrization=c("intercept"
 #' fit12_2 <- iterate_more(fit12)
 #' fit12_2
 #'
+#' # Structural GMVAR(1,2) model identified with sign
+#' # constraints. Only 10 iterations of the variable metric
+#' # algorithm
+#' W_122 <- matrix(c(1, -1, NA, 1), nrow=2)
+#' fit12s <- fitGMVAR(data, p=1, M=2, structural_pars=list(W=W_122),
+#'   ncalls=10, maxit=10, seeds=1:10)
+#' fit12s
+#'
+#' # Iterate more:
+#' fit12s_2 <- iterate_more(fit12s)
+#' fit12s_2
 #'
 #' # GMVAR(2,2) model with autoregressive parameters restricted
 #' # to be the same for all regimes, only 10 iterations of the
@@ -305,7 +377,8 @@ iterate_more <- function(gmvar, maxit=100, calc_std_errors=TRUE) {
   fn <- function(params) {
     tryCatch(loglikelihood_int(data=gmvar$data, p=gmvar$model$p, M=gmvar$model$M, params=params,
                                conditional=gmvar$model$conditional, parametrization=gmvar$model$parametrization,
-                               constraints=gmvar$model$constraints, check_params=TRUE, to_return="loglik",
+                               constraints=gmvar$model$constraints, structural_pars=gmvar$model$structural_pars,
+                               check_params=TRUE, to_return="loglik",
                                minval=minval), error=function(e) minval)
   }
   gr <- function(params) {
@@ -317,7 +390,8 @@ iterate_more <- function(gmvar, maxit=100, calc_std_errors=TRUE) {
 
   GMVAR(data=gmvar$data, p=gmvar$model$p, M=gmvar$model$M, params=res$par,
         conditional=gmvar$model$conditional, parametrization=gmvar$model$parametrization,
-        constraints=gmvar$model$constraints, calc_std_errors=calc_std_errors)
+        constraints=gmvar$model$constraints, structural_pars=gmvar$model$structural_pars,
+        calc_std_errors=calc_std_errors)
 }
 
 
